#!/usr/bin/env python

"""
Accessor classes for indexing.

Copyright (C) 2011 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class DelimitedRecord:

    "An accessor using a delimiter to split a record."

    def __init__(self, keys=None, delimiter=None, numeric=0):

        """
        Initialise the accessor using a sequence of 'keys' indicating the
        columns in each record that provide the values in the eventual compound
        key provided by each record, along with a 'delimiter' indicating how
        such columns are identified. If 'numeric' is set to a true value, keys
        will be interpreted as numbers.
        """

        self.keys = keys or [0]
        self.delimiter = delimiter
        self.numeric = numeric

        # Define a conversion method.

        self.convert = numeric and self.convert_numeric or (lambda x: x)

    def convert_numeric(self, term):
        return map(int, term)

    def get_key(self, record):
        values = record.split(self.delimiter)
        return self.convert([values[key] for key in self.keys])

    def get_sort_command(self):

        """
        Return the Unix sort command invocation required to produce the ordering
        described by this instance.
        """

        return "sort%s%s%s" % (
            self.delimiter and (" -t $'%s'" % repr(self.delimiter)[1:-1]) or "",
            self.numeric and " -n" or "",
            "".join([(" -k %d,%d" % (key + 1, key + 1)) for key in self.keys])
            )

# vim: tabstop=4 expandtab shiftwidth=4
